#define _GNU_SOURCE 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <pthread.h>
#include <sys/select.h>
#include "hpnsPlatform.h"
#include "hpnsUtil.h"
#include "tmodule.h"
#include "hpnsAppEngine.h"
#include "hpnsConfig.h"
#include "hpnsStm.h"
#include "hpnsHttpPost.h"
#include "httpDigest.h"
#include "hpnsNetwork.h"


#define tfree(x) { if ( x ) { free(x); x = 0; } }

extern int  hpnsEngineProcessMsgQueue(msg_t * param);
extern int  byteArrayToHexStr(char bytes[], int len, char hexstr[]) ;
extern void hpnsCleanUpTask();
extern int  hpnsInitTask();
extern int  hpnsUiProcessMsgQueue(msg_t *param);
extern int  hpnsInitUi(void *param);
extern void hpnsCleanUpUi();
extern volatile SNmsSocket pushServerFd;
extern volatile SNmsSocket hpnsPushTcpFd;


module_t      moduleObj[8];
sem_t         udpsem;
int           maxCid = 100;
int           msgID = 1;

extern  int  notificationRespFlag ;

typedef struct{
	UINT8  type;
	UINT8  method[10];
	UINT8  realm[100];
	UINT8  nonce[100];
	UINT8  opaque[100];
	UINT8  qop[100];
	UINT8  uri[100];
	UINT8  cnonce[100];
	UINT8  response[100];
	UINT8  nc[9];
	UINT8  userName[100];
	UINT8  passwd[100];	
}SHpnsHTTPAuthInfo;

typedef struct{
	UINT32 sessionId;
	UINT32 localSessionId;
	UINT32 recvLen;
	UINT32 checkSum;
	UINT8  hid[HPNS_HID_LEN];
}SHpnsTcpHeaderInfo;


//http auth info
#define HTTP_AUTH_INFO_REALM        "realm"
#define HTTP_AUTH_INFO_NONCE        "nonce"
#define HTTP_AUTH_INFO_URI          "uri"
#define HTTP_AUTH_INFO_QOP          "qop"
#define HTTP_AUTH_INFO_NC           "nc"
#define HTTP_AUTH_INFO_CNONCE       "cnonce"
#define HTTP_AUTH_INFO_RESPONSE     "response"
#define HTTP_AUTH_INFO_OPAQUE       "opaque"
#define HTTP_AUTH_INFO_USERNAME     "username"

//method
#define HTTP_METHOD_POST            "POST"
#define HTTP_METHOD_GET             "GET"



void wcToMbc (char *out, WCHAR *input)
{
	char *t;

	t = (char *)input;

	while ( *t != 0 )
	{
		*out ++ = *t++;
		t++;
	}

	*out++ = 0;
}

void mbsToWc (WCHAR *output, char *input) 
{
	int  i;
	char *t;

	t = (char *)output;

	while ( *input != 0 )
	{
		*t++ = *input++;
		*t++ = 0;
	}

	*t++ =0;
	*t++ = 0;
}

int hpnsWaitForTcpPkt(msg_t *param) 
{ 
    fd_set readFdSet ; 
    int ret=-1 ;
    SNmsSocket* nmsSocket = (SNmsSocket*)param->msg ;
    
    struct timeval   timeout;
    
    if (!nmsSocket) {
        nprintf("tcp thread is error with nmsSocket is NULL") ;
        return 0;
    }

    if (!hpnsSocketIsEqual(nmsSocket, (const SNmsSocket*)&hpnsPushTcpFd)) {
        nprintf("tcp thread sockFd:%d, incId: %d is not equal with hpnsPushTcpFd: %d, incId: %d", nmsSocket->sockFd, nmsSocket->incId, hpnsPushTcpFd.sockFd , hpnsPushTcpFd.incId) ;
        return 0;
    }

    timeout.tv_sec = 5 * 60;
    timeout.tv_usec = 0;

    FD_ZERO(&readFdSet);
    FD_SET(nmsSocket->sockFd, &readFdSet); 
    
    nprintf("tcp thread begin to select socket: %d, incId: %d", nmsSocket->sockFd, nmsSocket->incId);
    ret = select( nmsSocket->sockFd+1, &readFdSet, NULL, NULL, &timeout);

    if (ret < 0) {
        if (errno == EINTR) {
            nprintf("tcp select was intercept, so continue.");
            taosSendMsgToModule( &(moduleObj[3]), 0, 0, sizeof(SNmsSocket), (char*)nmsSocket);
        } else if (errno == EBADF) {
            nprintf("tcp select errono == EBADF, the socket may be close in other thread");
        } else {
            nprintf ("tcp select error, reason:%s, so close all tcp socket.", strerror(errno));
	     if(!hpnsSocketIsEqual(nmsSocket, (const SNmsSocket*)&hpnsPushTcpFd))
		hpnsSendMsgToEngine(HPNS_MSG_NW_ERROR_IND, (UINT8 *)nmsSocket, sizeof(SNmsSocket)); 
           }

        return 0 ;
    }

    if (ret == 0) {
        nprintf("tcp select time out, select next time");
        taosSendMsgToModule( &(moduleObj[3]), 0, 0, sizeof(SNmsSocket), (char*)nmsSocket);
        return 0;
    }

    /* ret > 0 */
    if (FD_ISSET(nmsSocket->sockFd, &readFdSet)) {
        if( hpnsSocketIsEqual(nmsSocket, (const SNmsSocket*)&hpnsPushTcpFd)) {
            nprintf("tcp socket: %d, incId: %d can read.", nmsSocket->sockFd, nmsSocket->incId);
            hpnsSendMsgToEngine(HPNS_MSG_DATA_READ_IND, (UINT8 *)nmsSocket, sizeof(SNmsSocket)); 
        } else {
            nprintf("tcp sockFd:%d, incId: %d can read now, but the hpnsPushTcpFd: %d, incId: %d is changed, the socket may be shutdown in other thread", nmsSocket->sockFd, nmsSocket->incId, hpnsPushTcpFd.sockFd , hpnsPushTcpFd.incId) ;
            // close(nmsSocket->sockFd) ;
        }
    }

    return 0;
} 


#ifdef __ANDROIDOS__
#define MOD_UPDLISTEN 1

int hpnsWaitForUdpPkt(msg_t *param) 
{ 
    fd_set fdSet, readFdSet, errorFdSet; 
    int ret = -1;
    SNmsSocket* nmsSocket = (SNmsSocket*)param->msg ;

    if (!nmsSocket) {
        nprintf("udp thread is error with nmsSocket is NULL") ;
        return 0;
    }

    if (!hpnsSocketIsEqual(nmsSocket, (const SNmsSocket*)&pushServerFd)) {
        nprintf("udp thread sockFd:%d, incId: %d is not equal with pushServerFd: %d, incId: %d", nmsSocket->sockFd, nmsSocket->incId, pushServerFd.sockFd , pushServerFd.incId) ;
        return 0;
    }

    
    FD_ZERO(&fdSet); 
    FD_SET(nmsSocket->sockFd, &fdSet); 
    readFdSet = fdSet; 
    errorFdSet = fdSet;

    nprintf("udp thread begin to select socket: %d, incId: %d", nmsSocket->sockFd, nmsSocket->incId);

    ret = select(nmsSocket->sockFd + 1, &readFdSet, 0, &errorFdSet, 0);

    if (ret < 0) {
        if (errno == EINTR) {
            nprintf("udp module listen errono == EINTR, continue it");
            taosSendMsgToModule( &(moduleObj[MOD_UPDLISTEN]), 0, 0, sizeof(SNmsSocket), (char*)nmsSocket);
        } else if (errno == EBADF) {
            nprintf("udp module listen errono == EBADF, the socket may be close in other thread");
        }
        else {
            nprintf ("udp module select error, reason:%s", strerror(errno));
            if (hpnsSocketIsEqual(nmsSocket, (const SNmsSocket*)&pushServerFd))
                hpnsSendMsgToEngine(HPNS_MSG_NW_ERROR_IND, (UINT8 *)nmsSocket, sizeof(SNmsSocket));
        }
        return 0 ;    
    }      

    if (ret == 0) {
        nprintf("udp module select ret == 0 may be time out and try again");
        taosSendMsgToModule( &(moduleObj[MOD_UPDLISTEN]), 0, 0, sizeof(SNmsSocket), (char*)nmsSocket);
        return 0 ;
    }

    /* ret > 0 */
    if(FD_ISSET(nmsSocket->sockFd, &errorFdSet)) {
        nprintf("udp module sockct error, so break and wait new connection, sockFd: %d, incId: %d, and pushServerFd: %d, incId: %d", nmsSocket->sockFd, nmsSocket->incId, pushServerFd.sockFd , pushServerFd.incId);
        if (hpnsSocketIsEqual(nmsSocket, (const SNmsSocket*)&pushServerFd))
            hpnsSendMsgToEngine(HPNS_MSG_NW_ERROR_IND, (UINT8 *)nmsSocket, sizeof(SNmsSocket));
        return 0;
    }
    
    if( FD_ISSET(nmsSocket->sockFd, &readFdSet) ) {
        if (!hpnsSocketIsEqual(nmsSocket, (const SNmsSocket*)&pushServerFd)) {
            nprintf("udp listen thread recv read signal, sockFd:%d, incId: %d is not equal with pushServerFd: %d, incId: %d, the socket need be close now", nmsSocket->sockFd, nmsSocket->incId, pushServerFd.sockFd , pushServerFd.incId) ;
            // close(nmsSocket->sockFd) ;
        } else {
            nprintf("udp listen thread recv read signal. socket: %d, incId: %d",  pushServerFd.sockFd , pushServerFd.incId);
            hpnsSendMsgToEngine(HPNS_MSG_DATA_READ_IND, (UINT8 *)nmsSocket, sizeof(SNmsSocket)); 
        }
    }
    
} 

#endif


int hpnsProcessUdpPkt(msg_t *param)
{
	pthread_attr_t attr;
	pthread_t       nsThread;

	pthread_attr_init (&attr);

	if ( pthread_create(&nsThread, &attr, (void *)hpnsWaitForUdpPkt, (void *)param) != 0 ) 
	{
		nprintf("failed to create process udp thread");
		
	}
}

int hpnsInitUdp(void *param)
{
	return 0;
}

void hpnsCleanUpUdp()
{

	return;
}

int hpnsInitTcp(void *param)
{
	return 0;
}

void hpnsCleanUpTcp()
{

	return;
}

int hpnsInitEng(void *param)
{
	return 0;
}

int nmsSystemDestroy()
{
	hpnsCloseConnectionToPushServer();
	return 0;
}

int nmSystemInit() 
{
  int i;
  
  moduleObj[0].name = "MMI";
  moduleObj[0].queueSize = 300;
  moduleObj[0].queueProtectSize= 200;
  moduleObj[0].processMsg = hpnsUiProcessMsgQueue;
  moduleObj[0].debugFlag = 3;
  moduleObj[0].init = hpnsInitUi; 
  moduleObj[0].cleanUp = hpnsCleanUpUi; 

  moduleObj[1].name = "UDP";
  moduleObj[1].queueSize = 300;
  moduleObj[1].queueProtectSize= 200;
  moduleObj[1].processMsg = hpnsWaitForUdpPkt;
  moduleObj[1].debugFlag = 3;
  moduleObj[1].init = hpnsInitUdp; 
  moduleObj[1].cleanUp = hpnsCleanUpUdp;
  
  moduleObj[2].name = "ENG";
  moduleObj[2].queueSize = 300;
  moduleObj[2].queueProtectSize= 200;
  moduleObj[2].processMsg = hpnsEngineProcessMsgQueue;
  moduleObj[2].debugFlag = 3;
  moduleObj[2].init = hpnsInitEng; 
  moduleObj[2].cleanUp = hpnsCleanUpTask; 

  moduleObj[3].name = "TCP";
  moduleObj[3].queueSize = 300;
  moduleObj[3].queueProtectSize= 200;
  moduleObj[3].processMsg = hpnsWaitForTcpPkt;
  moduleObj[3].debugFlag = 3;
  moduleObj[3].init = hpnsInitTcp; 
  moduleObj[3].cleanUp = hpnsCleanUpTcp; 

  for (i=0; i<4; ++i) {
      moduleObj[i].queueSize = maxCid;
      if ( taosInitModule( &moduleObj[i] ) < 0 )  
        return -1; 
	  printf ("module:%s is initialized\n", moduleObj[i].name);
  }

  //taosTmrInit (100, 500, 3600000);

  hpnsInitTask();

  return 0;
}

int nsHttpGetHeaderValue(char* pCont, UINT8 *value)
{
	if(!(pCont))
		return -1;

	char *pMsg = pCont;
	char *pStart = NULL, *pEnd = NULL;

	pStart = strchr(pMsg, '"');
	if(pStart == NULL)
	{
		pStart = strchr(pMsg, '=');
		if(pStart != NULL)
			pEnd = strchr(pMsg, ',');	
	}
	else
		pEnd = strchr(pStart+1, '"');	
	
	if(pStart == NULL || pEnd == NULL)
	{
		nprintf("failed to get value");
		return -1;
	}
	else
	{
		strncpy(value, pStart+1, pEnd - pStart - 1);
	}
	
	return 0;
}

int nsHttpGetAuthInfo(char* pMsgHeader, SHpnsHTTPAuthInfo *pHttpAuthInfo)
{
	if(!(pMsgHeader))
		return -1;

	const char *pMsg  = ( const char *)pMsgHeader;
	char *pCont = NULL;
	char ncStr[20] = {0};

	pCont = strcasestr(pMsg, (const char *)HTTP_AUTH_INFO_USERNAME);
	if(pCont)
		nsHttpGetHeaderValue(pCont, pHttpAuthInfo->userName);

	pCont = strcasestr(pMsg, (const char *)HTTP_AUTH_INFO_REALM);
	if(pCont)
		nsHttpGetHeaderValue(pCont, pHttpAuthInfo->realm);

	pCont = strcasestr(pMsg, (const char *)HTTP_AUTH_INFO_NONCE);
	if(pCont)
		nsHttpGetHeaderValue(pCont, pHttpAuthInfo->nonce);
	
	pCont = strcasestr(pMsg, (const char *)HTTP_AUTH_INFO_URI);
	if(pCont)
	nsHttpGetHeaderValue(pCont, pHttpAuthInfo->uri);
	
	pCont = strcasestr(pMsg, (const char *)HTTP_AUTH_INFO_CNONCE);
	if(pCont)
		nsHttpGetHeaderValue(pCont, pHttpAuthInfo->cnonce);

	pCont = strcasestr(pMsg, (const char *)HTTP_AUTH_INFO_RESPONSE);
	if(pCont)
		nsHttpGetHeaderValue(pCont, pHttpAuthInfo->response);

	pCont = strcasestr(pMsg, (const char *)HTTP_AUTH_INFO_OPAQUE);
	if(pCont)
		nsHttpGetHeaderValue(pCont, pHttpAuthInfo->opaque);

	nprintf("http auth info,method:%s, userName:%s, realm:%s, nonce:%s",pHttpAuthInfo->method, pHttpAuthInfo->userName,\
						pHttpAuthInfo->realm, pHttpAuthInfo->nonce);
	nprintf("url:%s, qop:%s, nc:%s, cnonce:%s, response:%s,opaque:%s",\
						pHttpAuthInfo->uri, pHttpAuthInfo->qop,\
						pHttpAuthInfo->nc, pHttpAuthInfo->cnonce, pHttpAuthInfo->response, pHttpAuthInfo->opaque);
	return 0; 
}


int hpnsSendMsgToNs(UINT32 appId, UINT8 msg[])
{
	int i =0, len, msgid = 0 ;
	char  *urlmsg = 0;
	char   buf[20000] = {0};
	SHpnsHTTPAuthInfo httpAuthInfo = {0};

	char  authInfo[1024] = {0};
	
	HASHHEX HA1;
	HASHHEX HA2 = "";
	HASHHEX Response;

	strcpy(httpAuthInfo.cnonce, "0a4f113b");
	strcpy(httpAuthInfo.method, "POST");
	strcpy(httpAuthInfo.nc, "00000001");
	strcpy(httpAuthInfo.qop, "auth");
	strcpy(httpAuthInfo.uri, "/index.html");
	strcpy(httpAuthInfo.userName, "test@126.com");
	strcpy(httpAuthInfo.passwd, "123654");
	
	
	for(i = 0; i < HPNS_MAX_BUNDLE_APP_NUM; i++)
	{
		if(hpnsInfo.appBundled[i].appId == appId && hpnsInfo.appBundled[i].status == 1)
		{
			break;
		}
	}

	if( i >= HPNS_MAX_BUNDLE_APP_NUM)
	{
		printf("failed to find app in app list");
		return -1;
	}
	
	char* params[10];
	int k;
	char  regStr[32] = {0};

	for(k = 0; k < 10; k++)
	{
		params[k] = calloc(1, 2000);
	}

	byteArrayToHexStr(hpnsInfo.appBundled[i].regId, 12, regStr);
	sprintf(params[0], "registration_id=%s", regStr);

	msgid = ++msgID;
	sprintf(params[1], "message_id=%d", msgid);

	sprintf(params[2], "expiry=300");

	urlmsg = urlencode((char const*)msg, strlen(msg), &len);
	sprintf(params[3], "payload=%s", urlmsg);

	char *ret = postURL("http://118.26.192.202:7568", NULL, NULL, NULL, 0, (char **)&buf, params, 4);
	//nprintf("post return buf:%s\n", buf);
	nprintf("post url return cont:%s\n", ret ? ret : " ");

	if(ret && strcasecmp(ret, "200 OK") == 0)
	{
		nprintf("send msg to ns successfully\n");
	}
	else
	{
		nsHttpGetAuthInfo(buf, &httpAuthInfo);
		DigestCalcHA1("md5",httpAuthInfo.userName, httpAuthInfo.realm, httpAuthInfo.passwd, httpAuthInfo.nonce, httpAuthInfo.cnonce, HA1);
		nprintf("HA1:%s", HA1);
		DigestCalcResponse(HA1, httpAuthInfo.nonce, httpAuthInfo.nc, httpAuthInfo.cnonce, httpAuthInfo.qop,\
			httpAuthInfo.method, httpAuthInfo.uri, HA2, Response);
		nprintf("response:%s", Response);

		sprintf(authInfo, "username=\"%s\",realm=\"%s\",nonce=\"%s\",uri=\"%s\",qop=\"%s\",nc=%s,cnonce=\"%s\",response=\"%s\",opaque=\"%s\"",\
			httpAuthInfo.userName, httpAuthInfo.realm, httpAuthInfo.nonce, httpAuthInfo.uri, httpAuthInfo.qop, httpAuthInfo.nc, httpAuthInfo.cnonce,\
			Response, httpAuthInfo.opaque);

		sprintf(params[0], "registration_id=%s", regStr);
		sprintf(params[1], "message_id=%d", msgid);
		sprintf(params[2], "expiry=300");
		sprintf(params[3], "payload=%s", urlmsg);
		
		ret = postURL("http://118.26.192.202:7568", NULL, NULL, authInfo, 0, (char **)&buf, params, 4);
		nprintf("post url return cont:%s\n", ret ? ret : " ");
		if(ret && strcasecmp(ret, "200 OK") == 0)
		{
			nprintf("send msg to ns successfully\n");
		}
	}

	for(k = 0; k < 10; k++)
	{
		tfree(params[k]);
	}

	return 0;
}


void requstToken(unsigned int appId, char senderId[])
{
	SHpnsRegInfo appprofile = {0};
	appprofile.appId = appId;
	strncpy(appprofile.senderId, senderId, HPNS_SENDER_LEN);
	hpnsSendMsgToEngine(HPNS_MSG_REG_REQ, (UINT8 *)(&appprofile), sizeof(SHpnsRegInfo));

	return;
}

int main (int argc, char *argv[])
{
	char     c;
	SHpnsRegInfo appprofile = {0};
	UINT32   appId = 0, i = 0, ret;
	char     msg[1024] = {0};
	UINT8    regid[12]= {0};

	if ( nmSystemInit() < 0 )
	{
		printf ("failed to initialize system, exit\n");
		exit(0);
	}
	
	c = 1;
	
	while ( 1 )
	{
		switch ( c ) 
		{
		
		case 'a':
			printf("Please input your appId:\n");
			scanf("%d",&(appprofile.appId));
			printf("Please input your app sender id:\n");
			scanf( "%s", appprofile.senderId);
			printf("appId:%d  senderId:%s\n", appprofile.appId, appprofile.senderId );
			hpnsSendMsgToEngine(HPNS_MSG_REG_REQ, (UINT8 *)(&appprofile), sizeof(SHpnsRegInfo));
			break;
			
		case 'b':
			printf("Please input your appId:\n");
			scanf("%d",&appId);
			nprintf("unbind appid:%d", appId);
			hpnsSendMsgToEngine(HPNS_MSG_UNREG_REQ, (UINT8 *)appId, 0);
			break;
			
		case 'c':
			printf("send a notification to ns,please input appId: and msg\n");
			scanf("%d",&appId);
			gets(msg);
			printf("appId:%d, msg:%s\n", appId, msg);
			hpnsSendMsgToNs(appId, msg);
			break;

		case 'd':
			memset(&appprofile, 0x0, sizeof(SHpnsRegInfo));
			printf("Please input second appId:\n");
			scanf("%d",&(appprofile.appId));
			printf("Please input second app sender id:\n");
			scanf( "%s", appprofile.senderId);
			printf("appId:%d  senderId:%s\n", appprofile.appId, appprofile.senderId );
	
			hpnsSendMsgToEngine(HPNS_MSG_REG_REQ, (UINT8 *)(&appprofile), sizeof(SHpnsRegInfo));

			appprofile.appId = 1;
			strcpy(appprofile.senderId, "xitang@126.com");
			hpnsSendMsgToEngine(HPNS_MSG_REG_REQ, (UINT8 *)(&appprofile), sizeof(SHpnsRegInfo));
			break;

		case 'e':
			printf("Please input error your appId:\n");
			scanf("%d",&appId);

			hpnsSendMsgToEngine(HPNS_MSG_UNREG_REQ, (UINT8 *)appId, 0);
			break;

		case 'f':
			memset(&appprofile, 0x0, sizeof(SHpnsRegInfo));
			appprofile.appId = 2;
			hpnsSendMsgToEngine(HPNS_MSG_REG_REQ, (UINT8 *)(&appprofile), sizeof(SHpnsRegInfo));
			break;

		case 'g':
			appprofile.appId = 100;
			hpnsSendMsgToEngine(HPNS_MSG_REG_REQ, (UINT8 *)(&appprofile), sizeof(SHpnsRegInfo));
			break;

		case 'h':
			hpnsSendMsgToEngine(HPNS_MSG_UNREG_REQ, (UINT8 *)2, 0);
			break;
		
		case 'l':
			printf("\n====appId====== senderId ========status======regId===========\n");
			for(i = 0; i < HPNS_MAX_BUNDLE_APP_NUM; i++)
			{

				if(hpnsInfo.appBundled[i].appId != 0)
				{
					printf("    %d         %s        %d    %05d-%05d-%010d-%010d\n",hpnsInfo.appBundled[i].appId,\
						hpnsInfo.appBundled[i].senderId, hpnsInfo.appBundled[i].status, \
						*(UINT16 *)(hpnsInfo.appBundled[i].regId), *(UINT16 *)(hpnsInfo.appBundled[i].regId+2),\
						*(UINT32 *)(hpnsInfo.appBundled[i].regId+4), *(UINT32 *)(hpnsInfo.appBundled[i].regId+8));

				}

			}
			break;

		case 'x':
			printf ("thanks for using HPNs linux version\n");
			exit(0);
			
		case 'm':
			printf("\n========HPNs linux version======\n");

			printf("a: boundle a application\n");
			printf("b: unbundle a application\n");
			printf("c: send a notification\n");
			printf("l: list all app in app list\n");
			printf("\n");
			printf("x: exit hesine linux version\n");
			printf("\n\n");
			break;
			
		default:
			
			break;
			
		}

		c = getchar();
	}	

}



